## ==============================================================
## * Add methods currently not present in Matrix package
## * Improve performance of som Matrix methods
## ==============================================================

## Subset-of-inverse
## A: "dsCMatrix" with a "CHMfactor".
## SigmaStart: Symmetric "dgCMatrix" containing a pre-computed lower
## right corner of the result  - in the permuted domain!.
factorToSparse <- function(A).Call("CHMfactor_to_sparse",A,PACKAGE="Matrix")
setGeneric("solveSubset",
           function(A,...)standardGeneric("solveSubset"))
setMethod("solveSubset",signature("CHMfactor"),
          function(A,SigmaStart=NULL){
            .Call("inv_up",factorToSparse(A),
                  A@perm,SigmaStart)
          })
setMethod("solveSubset",signature("dsCMatrix"),
          function(A,L=A@factors[[1]]){
            if(length(A@factors)==0)stop("dsCMatrix has no factors")
            solveSubset(L)
          })
setMethod("solveSubset",signature("dtCMatrix"),
          function(A,perm=0:(nrow(A)-1),SigmaStart=NULL){
            if(A@uplo=="U")A <- t(A)
            .Call("inv_up",A,as.integer(perm),SigmaStart)
          })
setMethod("solveSubset",signature("dgCMatrix"),
          function(A,perm=0:(nrow(A)-1),SigmaStart=NULL){
            ## WARNING: SAFETY CHECK TURNED OFF BECAUSE OF PERFORMANCE
            ##A <- as(A,"dtCMatrix")
            ##solveSubset(A,perm,SigmaStart)
            .Call("inv_up",A,as.integer(perm),SigmaStart)
          })


## Fast sparse matrix addition
local({
  for(sig in list(signature("dsCMatrix","dsCMatrix"),
                  signature("dsCMatrix","dgCMatrix"),
                  signature("dgCMatrix","dsCMatrix"),
                  signature("dgCMatrix","dgCMatrix")
                  )){
    setMethod("+",sig,
              function(e1,e2).Call("add_sparse",e1,e2,PACKAGE="lgc"))
    setMethod("-",sig,
              function(e1,e2).Call("add_sparse",e1,-e2,PACKAGE="lgc"))
  }  
})

## Fast conversion dgCMatrix -> dsCMatrix
## Gives segmentation faults in R-2.11.1
setAs("dgCMatrix","dsCMatrix",function(from,to).Call("dgC2dsC",from,PACKAGE="lgc"))

## <matrix> * <dgCMatrix>
setMethod("*",signature(e1="matrix",e2="dgCMatrix"),
          function(e1,e2){
            e2 <- as(e2,"dgTMatrix")
            e2@x <- e1[cbind(e2@i,e2@j)+1]*e2@x
            as(e2,"dgCMatrix")
          })

## Fast Kronecer avoiding triplet conversion.
setMethod("kronecker",signature(X="dgCMatrix",Y="dgCMatrix"),
          function(X,Y,FUN = "*", make.dimnames = FALSE, ...){
            .Call("kronecker",X,Y)
          })
## Hack to avoid using these methods:
## * selectMethod("kronecker",c("dgCMatrix","dsCMatrix"))
## * selectMethod("kronecker",c("dsCMatrix","dgCMatrix"))
setMethod("kronecker",
          signature("dgCMatrix","dsCMatrix"),
          function(X,Y, FUN = "*", make.dimnames = FALSE,...){
            kronecker(X,as(Y,"dgCMatrix"))
          })



## ---------------- 2x2 block of sparse matrices
## [A C]
## [B D]
block2x2 <- function(A,B,C,D){
  cbind2( rbind2(A,B) , rbind2(C,D) )
}
## [A t(B)]
## [B D   ]
symblock2x2 <- function(A,B,C){
  cbind2( rbind2(A,B) , rbind2(t(B),C) ) 
}
## [A 0]
## [B C]
triblock2x2 <- function(A,B,C){
  #Z <- Matrix(0,nrow(A),ncol(C))
  Z <- .Call("spzero_matrix",nrow(A),ncol(C),PACKAGE="lgc")
  .Call("Csparse_horzcat",
        .Call("Csparse_vertcat",A,B) ,
        .Call("Csparse_vertcat",Z,C) )  
}

diagblock2x2 <- function(A,B){
  Z <- .Call("spzero_matrix",nrow(A),ncol(B),PACKAGE="lgc")
  Zt <- .Call("spzero_matrix",nrow(B),ncol(A),PACKAGE="lgc")
  .Call("Csparse_horzcat",
        .Call("Csparse_vertcat",A,Zt) ,
        .Call("Csparse_vertcat",Z,B) )    
}

ZeroMatrix <- function(Dim){
  Dim <- as.integer(Dim)
  .Call("spzero_matrix",Dim[1],Dim[2],PACKAGE="lgc")
}

cornerMatrix <- function(n,x){
  n <- as.integer(n)
  new("dgCMatrix",x=x@x,i=x@i+n,p=c(rep.int(0L,n),x@p),Dim=x@Dim+n)
}

blockToList <- function(x,row,col){
  i1 <- 0:(row-1)
  i2 <- row:(nrow(x)-1)
  j1 <- 0:(col-1)
  j2 <- col:(ncol(x)-1)
  submat <- function(i,j).Call("Csparse_submatrix",x,i,j,PACKAGE="Matrix")
  ans <- list(submat(i1,j1),submat(i2,j1),submat(i1,j2),submat(i2,j2))
  dim(ans) <- c(2,2)
  ans
}

sparseSubset <- function(x,i,j)
  .Call("Csparse_submatrix",x,as.integer(i-1),as.integer(j-1),PACKAGE="Matrix")

setMethod("*",signature("dgCMatrix","dgCMatrix"),
          function(e1,e2){
            if(!identical(dim(e1),dim(e2)))stop("mul_sparse only works for matrices of equal dimension.")
            .Call("mul_sparse",e1,e2,PACKAGE="lgc")
          })

mybdiag <- function(qw){
  n <- length(qw)

  ## For now: dense matrices -> dgCMatrix
  for(i in seq(n))if(!is(qw[[i]],"sparseMatrix"))
    qw[[i]] <- as(qw[[i]],"dgCMatrix")

  xlen <- unlist(lapply(qw,function(x)length(x@x)))
  nrow <- unlist(lapply(qw,function(x)x@Dim[1]))
  ncol <- unlist(lapply(qw,function(x)x@Dim[2]))
  
  ## - Assume all matrices have identical class and storage!
  ## - Ignore dimnames
  cl <- class(qw[[1]])

  ## x
  x <- unlist(lapply(qw,function(x)x@x))
  
  ## i
  tmp <- c(0,cumsum(nrow)[-n])  
  i <- (lapply(qw,function(x)x@i))
  i <- unlist(Map("+",i,tmp))

  ## p
  tmp <- c(0,cumsum(xlen)[-n])
  p <- (lapply(qw,function(x)x@p[-length(x@p)]))
  p <- unlist(Map("+",p,tmp))
  p <- c(p,sum(xlen))

  ## Dim
  dim <- c(sum(nrow),sum(ncol))

  switch(cl,
         "dgCMatrix"=new(cl,x=x,i=as.integer(i),p=as.integer(p),Dim=as.integer(dim)),
         "dsCMatrix"=new(cl,x=x,i=as.integer(i),p=as.integer(p),Dim=as.integer(dim),uplo=qw[[1]]@uplo),
         "dtCMatrix"=new(cl,x=x,i=as.integer(i),p=as.integer(p),Dim=as.integer(dim),uplo=qw[[1]]@uplo,diag=qw[[1]]@diag) )
}


## Methods missing in Matrix package R-2.11.1
setMethod("cbind2",c("dgCMatrix","dgeMatrix"),function(x,y)cbind2(x,as(y,"dgCMatrix")))
setMethod("+",c("dgCMatrix","matrix"),function (e1, e2)as(e1,"matrix")+e2 )
setMethod("kronecker",c("sparseMatrix","matrix"),
          function (X, Y, FUN = "*", make.dimnames = FALSE, ...)
          callGeneric(X,as(Y,"dgCMatrix")) )
