## Interface to lgc_sparse.c
.lgc_sparse <- function(N,covstruct,perm,etafix=NULL,etastart=NULL,...){
  beSilent <- !as.logical(.lgc_control()@trace)
  data2sparse <- function(v){
    i <- which(is.finite(v))
    as(spMatrix(length(v),1,i=i,j=rep(1,length(i)),v[i]),"dgCMatrix")
  }
  ##mu <- rep(0,length(N))
  Q <- evaluate(covstruct)
  usePerm <- if(!missing(perm))(length(perm)==nrow(Q)) else FALSE
  if(usePerm) .lgc_symbolic(Q,perm) else .lgc_symbolic(Q)
  N <- data2sparse(N)
  .Call("lgc_setN",N)
  .Call("lgc_setetastart",etastart) ## log(N+.1)
  if(!is.null(etafix)){
    if(!is(etafix,"sparseMatrix"))etafix <- data2sparse(etafix)
    .Call("lgc_setetafix",etafix)
  }
  fun <- function(thetav,gradient=FALSE,report=FALSE,...){
    .lgc_derivmode(gradient)
    covenv <- try(do.call(covstruct@fun, as.list(thetav)),silent=beSilent)
    if(is(covenv,"try-error"))return(NaN)
    ok <- is(try(covenv$precision),"sparseMatrix")
    if(!ok)return(NaN)
    .Call("lgc_setQ",covenv$precision)
    ok <- is.null(try(.Call("lgc_newton"),silent=FALSE))
    if(!ok)return(NaN)
    if(is.logical(report)){
      if(report)return(list(value=as.numeric(.Call("lgc_getvalue")),
                            eta=as.numeric(.Call("lgc_geteta"))))
    }
    if(!gradient)return (as.numeric(.Call("lgc_getvalue")))
    if(gradient)return (
#                        c(mu,
                        as.numeric(.Call("lgc_gradient",covenv$Dprecision)) )
  }
  rm(Q)
  fun
}
## See lgc_sparse.c
setClass("lgc_control",
         representation(trace="integer",tol="numeric",mgcmax="numeric",maxit="integer",report="integer",
                        nrelax="integer",zrelax="numeric",countflops="integer",
                        experimental="integer",externallib="integer",penalty="numeric"),
         prototype(trace=as.integer(1),
                   tol=as.numeric(1e-8),
                   mgcmax=as.numeric(1e15),
                   maxit=as.integer(50),
                   report=as.integer(1),
                   nrelax=as.integer(c(4,16,48)),
                   zrelax=as.numeric(c(0.8,0.1,0.05)),
                   countflops=as.integer(1),
                   experimental=as.integer(0),
                   externallib=as.integer(0),
                   penalty=as.numeric(1e4))
         )

##' Control parameters for fitlgc.
##'
##' Adjust global lgc options in \code{options("lgc_control")}.
##'
##' Currently defined options:
##' \itemize{
##' \item trace Print tracing information during model fitting, in particular (1) summary of symbolic analysis (2)
##' progression of the inner newton iterations.
##' \item tol Maximum gradient convergence criterion for inner newton iterations.
##' \item mgcmax Drop out of inner newton optimizer and return NaN if inner gradient is too steep (because this
##' indicates that the outer parameter is crazy).
##' \item maxit Maximum number of iterations for inner newton optimizer.
##' \item report Calculate gradient of \eqn{\eta} wrt. \eqn{\theta}?
##' \item nrelax Supernode tolerance - see CHOLMOD manual.
##' \item zrelax Supernode tolerance - see CHOLMOD manual.
##' \item countflops Empirical flopcount of inverse subset algorithm?
##' \item experimental Control experimental features - see source code.
##' \item externallib Use external version of CHOLMOD to do the symbolic analysis. Reason: The
##' \code{Matrix} package does not have the METIS orderings. For large 3D fields this option can often
##' reduce the flopcount with a factor of 10.
##' \item penalty Control the precision of soft constraint used to fix random effects.
##' }
##' @param ... Control arguments - see details.
##' @return Object of class \code{lgc_control}
##' @rdname lgc_control
.lgc_control <- function(...){
  con <- getOption("lgc_control") ## First check if allready set
  if(is.null(con))con <- new("lgc_control",...)
  args <- list(...)
  nm <- names(args)
  for(i in seq(args))slot(con,nm[i]) <- args[[i]]
  .Call("lgc_setcontrol",new("lgc_control",con))
  options(lgc_control=con)
  con
}
.lgc_cleanup <- function(){
  .Call("lgc_cleanup")
}
.lgc_symbolic <- function(Q,p=NULL).Call("lgc_symbolic",Q,p)

