## North Sea Cod 2009 Q1. Fit first order Gaussian Markov random field with nugget effect. Plot estimated landscape. Compare with spatial exponential correlation model.
require(lgc)
data(NorthSeaCod.2009.Q1)

## Construct grid and attach nearest gridpoint to every observation
grid <- gridConstruct(NorthSeaCod.2009.Q1,nearestObs=100)
NorthSeaCod.2009.Q1$position <- gridFactor(NorthSeaCod.2009.Q1,grid)

## Fit first order Gaussian Markov random field with nugget effect
fit <- fitlgc(N~PARM("logscale")*GMRF1(position),fixef=~1,nugef=~1,
               data=subset(NorthSeaCod.2009.Q1,sizeGroup=="(50,60]"))

## Parameters
print(fit)
summary(fit)

## Spatial landscape
fv <- fitted(fit) ## Fitted random effects
lapply(fv,dim) ## Structure of random effect output
image(grid,as.vector(fv[[2]])) ## Component 2 contains spatial part

## ================================================================
## Alternative model: Spatial exponential correlation with nugget effect.
## Use "COV" in formula interface. Available distance matrices are
## given in .lgc_distances 
## ================================================================
fit.exp <- fitlgc(N~COV(position,~exp(loga)*exp(-exp(logb)*dist.km),
                         parameters=c(loga=0,logb=-4)),fixef=~1,nugef=~1,
                   data=subset(NorthSeaCod.2009.Q1,sizeGroup=="(50,60]"))
fv2 <- fitted(fit.exp) ## Fitted random effects
lapply(fv2,dim) ## Structure of random effect output
image(grid,as.vector(fv2[[2]])) ## Component 2 contains spatial part

