## Binomial-multivariate-lognormal distribution:
## How to fit 0-1-data using the lgc-model.

## We use the following property of the Poisson distribution:
## If X~Pois(lambda) and Y~Pois(1) then
## X|X+Y ~ Bin(X+Y,inv.logit(log(lambda)))
## The trick:
## Given observations with intensity lambda(x)
## obs:            0 1 0 0 1 1 1 0 1 1 1 1 1 1 1 1 1 1 1 1
## Add pseudo observations to dataset:
## pseudo (1-obs): 1 0 1 1 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0
## Add fixed effect intercept="observation id" such that pairs
## of (obs,pseudo obs) are grouped together.
## Use as estimate p(x)=lambda(x)/(1+lambda(x))=inv.logit(eta)

## Test it:
require(lgc)
set.seed(123);Q <- evaluate(covStructAR(c(phi=.9),n=200))
eta <- 3*as.vector(GMRFsample(Q))
prob <- binomial()$linkinv(eta)

## Observations
d <- data.frame(n=rbinom(length(prob),1,prob),x=factor(seq(prob)))

## Augment the data.frame:
## * Pseudo observations = 1-obervations
## * Place NAs on all explanatory variables
## * Add factor that groups observations with pseudo-observations.
d1 <- d; d1$id <- factor(1:nrow(d1))
d2 <- d1
d2$n <- 1-d1$n
d2$x[] <- NA
dd <- rbind(d1,d2)

## Fit model
fit <- fitlgc(n~PARM("logscale")*AR1(x)+FIXEF(id),fixef=NULL,data=dd)
fv <- fitted(fit)
plot(prob)
points(d$n,col="red")
points(binomial()$linkinv(fv[[2]]),col="red",type="l")

## Parameter estimates
summary(fit)
f <- function(x) 2/(1 + exp(-2 * x)) - 1
par <- as.numeric(fit$par)
print(c(scale=exp(par[1]),phi=f(par[2])))

## For comparison one might consider the *wrong* poisson model:
## fit2 <- fitlgc(n~PARM("logscale")*AR1(x),fixef=NULL,data=d)
