## Formula interface to fit 2D separable field on 50x50 grid.

library(lgc)
set.seed(123)
##n <- 10
##n <- 20
n <- 50
##n <- 100
##n <- 200
##n <- 500

## alpha=-log(phi)
## Q0 <- covStructAR(c(phi=.5),n=50)
phi1=exp(-1/(.1*n)) ## Correlation range=10% of grid size first dimension
phi2=exp(-1/(.2*n)) ## Correlation range=20% of grid size second dimension

## Make sparce precision of stationary AR(1) processes
Q1 <- covStructAR(c(phi1=phi1),n=n)
Q2 <- covStructAR(c(phi2=phi2),n=n)

## Kronecker product
Q <- Q1%x%Q2

## Simulate data
Qtmp <- evaluate(Q)
eta <- GMRFsample(Qtmp)
dim(eta) <- c(n,n)
image(eta)
N <- rpois(nrow(Qtmp),exp(eta))
##d <- data.frame(N=N)
d <- expand.grid(x=factor(1:n),y=factor(1:n))
d$N <- N

## Fit model -  Known mean=0 and variance=1
fit <- fitlgc(N~AR1(y)*AR1(x),data=d,fixef=NULL)
fv <- fitted(fit)
image(fv[[2]])

## Fit model -  Unnown mean and variance=1
fit2 <- fitlgc(N~AR1(y)*AR1(x),data=d,fixef=~1)
fv <- fitted(fit2)
image(fv[[2]])

## Fit model -  Unnown mean and unknown variance
fit3 <- fitlgc(N~PARM("logvar")*AR1(y)*AR1(x),data=d,fixef=~1)
fv <- fitted(fit3)
image(fv[[2]])

