## Formula interface to implement non-linear mean value structure.

library(lgc)
set.seed(123)
n <- 2500


## Use small correlation range - 1/2% of grid size
## -otherwise difficult to identify non-linear terms
phi=exp(-1/(.005*n)) 

## Make sparse precision of stationary AR(1) processes
Q <- covStructAR(c(phi=phi),n=n)

## Simulate data
Qtmp <- evaluate(Q)
eta <- GMRFsample(Qtmp)
plot(eta)

## covariates
z <- seq(0,1,length=n)

## Case 1: linear function
funz <- 5*z+3
eta2 <- eta+funz
N <- rpois(nrow(Qtmp),exp(eta2))
d <- data.frame(x=factor(1:n),z=z,N=N)

.lgc_control(tol=1e-6,mgcmax=1e15,penalty=1e+4,trace=0L)
fit <- fitlgc(N~AR1(x),data=d,fixef=NULL,nonlin=~a0*z+a1,parameters=c(a0=0,a1=0),delta=1e-4,parscale=c(1,1,1))
summary(fit)

## comparison
fit. <- fitlgc(N~AR1(x),data=d,fixef=~1,nonlin=~a0*z,parameters=c(a0=0),delta=1e-4,parscale=c(1,1))
summary(fit.)
fit.. <- fitlgc(N~AR1(x),data=d,fixef=~z,nonlin=NULL,delta=1e-4,parscale=c(1))
summary(fit..)

## Case 2: non linear function
funz <- log(1/(1+exp(-10*(z-0.5))))
eta2 <- eta+funz
N <- rpois(nrow(Qtmp),exp(eta2))
d <- data.frame(x=factor(1:n),z=z,N=N)

.lgc_control(tol=1e-6,mgcmax=1e15,penalty=1e+4)
fit2 <- fitlgc(N~AR1(x),data=d,fixef=NULL,nonlin=~-log(1+exp(-a0*(z-a1))),parameters=c(a0=0,a1=.2),delta=1e-4,parscale=c(.1,.1,.1))
summary(fit2)


