\name{covStructFormula}
\alias{covStructFormula}
\title{Formula interface to covStructs.}
\usage{
  covStructFormula(formula, data, fixef = NULL,
    delta = 1e-06, invdelta = 1/delta,
    delta.nonlin = delta, nugef = NULL, update.args = NULL,
    scale = NULL, nonlin = NULL, parameters = NULL,
    denseFixef = FALSE, ignore.proj = FALSE)
}
\arguments{
  \item{formula}{Random effect formula (details below).}

  \item{data}{Data.frame}

  \item{fixef}{Fixed effect formula. NAs in model.matrix
  are replaced by zeros.}

  \item{delta}{Tiny number. Precision of flat prior on
  fixed effects.}

  \item{invdelta}{Huge number. Precision of nugget effect
  when no nugget is desired.}

  \item{nugef}{Nugget effect formula.}

  \item{update.args}{List used to re-parameterize the
  covStruct.}
}
\value{
  Object of class covStruct with attributes used by
  \code{fitlgc}.
}
\description{
  Specify a full Gaussian Markov Random Field via formulas.
}
\details{
  The present interface allow the construction of Gaussian
  Random Fields of the form \eqn{Z=Z'+A \beta} where
  \eqn{Z'=B Y+\epsilon} and \eqn{Y=c(X_1,...,X_k}). With
  this notation \eqn{Y} is the mean zero hidden random
  field which is a composition of multiple hidden
  multivariate layers \eqn{X_1,...,X_k}. The design matrix
  \eqn{B} plays the role of looking up the relevant
  contributions of \eqn{X_1,...,X_k} to each observation
  entry. The construction is controlled by the arguments
  \itemize{ \item \code{formula} Generates the Individual
  processes \eqn{X_1,...,X_k} and the design matrix
  \eqn{B}.  \item \code{fixef} Generates fixed effect
  design matrix \eqn{A}.  \item \code{nugef} Generates the
  nugget effect \eqn{\epsilon}. } Currently defined syntax
  of the random field \code{formula} \enumerate{ \item
  \code{ID(factor)} Defines IID process with variance one
  on the levels of \code{factor}. \item \code{DIAG(factor)}
  Defines a process of independent variables with a
  variance parameter for each level of \code{factor}. \item
  \code{AR1(factor)} Defines a stationary AR1 process with
  variance one on the levels of \code{factor}. \item
  \code{AR2(factor)} Defines a stationary AR2 process with
  variance one on the levels of \code{factor}. \item
  \code{contAR2(numeric)} Defines a variance 1 process with
  covariance structure
  \eqn{\rho(t)=\exp(a*t/s)*(\cos(b*t/s)-a/b*\sin(b*t/s))},
  \eqn{b=\sqrt{1+a^2}} (with derivative augmentation to
  obtain exact sparseness of the precision matrix on any
  irregular grid). The parameter \eqn{a} is a shape
  parameter in the interval \eqn{(-1,0)} and \eqn{s} is a
  positive scale parameter. The structure solves a linear
  2nd order ODE and can approximate most powered
  exponential correlations. For instance the shape
  \eqn{a=-0.77} is close to a Gaussian correlation. \item
  \code{CORR(factor)} Defines a free un-structured
  correlation matrix on the levels of \code{factor}. \item
  \code{GMRF1(gridFactor)} Defines a first order Gaussian
  Markov Random Field on the grid associated with
  \code{gridFactor}.  Is neither stationary nor of variance
  one. The model is also known as "Besags proper model".
  \item \code{COV(distFactor,form)} Defines a process in
  terms of the covariance specified through the formula
  \code{form}.  Distance matrices (or increment matrices)
  are looked up in the level attributes of
  \code{distFactor}. \item \code{*} The notation \code{X*Y}
  generates a process on the product space with the
  multiplicative covariance structure. \item \code{+} The
  notation \code{X+Y} generates independent processes X and
  Y and relates the observation to the sum. \item \code{:}
  The notation \code{a:X} relates the observation to the
  process X scaled with a covariate a. \item
  \code{PARM(name)} Generates a scale parameter that can be
  multiplied with a term. \item \code{SCALE(X)} Multiplies
  different scale parameters on each component of the
  process \code{X} (should only be applied to reasonably
  low dimensional \code{X}).  The aggregation of scale
  parameters can be controlled by \code{SCALE(X,fac)} where
  \code{fac} is a factor with length equal to the dimension
  of \code{X}. This will add \code{nlevels(fac)} scale
  parameters. \item \code{PROJ(X)} Project the covariance
  structure by removing entries for which no direct
  data-entry is associated. This does not change the model
  but only affects the internal sparse representation and
  computation time. Note that projection generally destroys
  sparseness. However it can sometimes improve performance
  on large separable fields by replacing e.g \code{X*Y*Z}
  by \code{X*Y*PROJ(Z)}. } Note that factors with NAs can
  be used to indicate that there is no relation between a
  given observation and one or more of the random field
  terms. This may be useful in situations where multiple
  different data sources are modelled.
}
\examples{
##' === Separable correlation on grid with a nugget effect.
d <- expand.grid(x=factor(1:7),y=factor(1:8))
C <- covStructFormula(~AR1(x)*AR1(y)+AR2(y),nugef=~1,fixef=NULL,data=d)
##' === Some methods
C@parameters
simulate(C)
image(C)
}
\seealso{
  \code{\link{fitlgc}}
}

