#include <R_ext/Rdynload.h>
#include <Mutils.h>
#include "Matrix.h"

cholmod_common c;
void R_init_lgc(DllInfo *dll)
{
    M_R_cholmod_start(&c);
}
void R_unload_lgc(DllInfo *dll)
{
    M_cholmod_finish(&c);
}

SEXP add_sparse(SEXP xp, SEXP yp){
  
  CHM_SP x=AS_CHM_SP(xp);
  CHM_SP y=AS_CHM_SP(yp);
  double one[] = {1, 0};
  CHM_SP z=M_cholmod_add(x, y, one, one, TRUE, TRUE, &c);
  return M_chm_sparse_to_SEXP(z, 1, 0, 0, "", R_NilValue);
}

SEXP spzero_matrix(SEXP nrow, SEXP ncol){
  CHM_SP x=M_cholmod_allocate_sparse(INTEGER(nrow)[0], INTEGER(ncol)[0], 
				     0 /*nzmax*/,
				     TRUE, TRUE,  0 /*stype*/,
				     CHOLMOD_REAL /*xtype*/, &c);
  return M_chm_sparse_to_SEXP(x, 1, 0, 0, "", R_NilValue);
}

/* Sparse kronecker product */
SEXP kronecker(SEXP x, SEXP y){
  
  CHM_SP A=AS_CHM_SP(x);
  CHM_SP B=AS_CHM_SP(y);
  int Am=A->nrow, Bm=B->nrow, An=A->ncol, Bn=B->ncol, Anzmax=A->nzmax, Bnzmax=B->nzmax;
  CHM_SP C=M_cholmod_allocate_sparse(Am*Bm, An*Bn, Anzmax*Bnzmax,TRUE/*sorted*/,
				     TRUE/*packed*/,0 /*stype*/,CHOLMOD_REAL /*xtype*/,&c);

  int i,j,p,q,k=0,l=0;
  double *Ax=A->x,*Bx=B->x,*Cx=C->x;
  int *Ai=A->i,*Bi=B->i,*Ci=C->i,*Ap=A->p,*Bp=B->p, *Cp=C->p;
  /* The (i,j)-column of C contains "outer(B[,j],A[,i])" */
  for(i=0;i<An;i++) /* loop over columns (i,j) of C */
    for(j=0;j<Bn;j++){
      Cp[l]=k;      /* k=current non-zero position */
      l++;            /* l=current column of C */
      for(p=Ap[i];p<Ap[i+1];p++)
	for(q=Bp[j];q<Bp[j+1];q++){
	  Cx[k]=Ax[p]*Bx[q];
	  Ci[k]=Ai[p]*Bm+Bi[q];
	  k++;
	}
    }
  Cp[l]=k; /* Finalize p */

  return M_chm_sparse_to_SEXP(C, 1, 0, 0, "", R_NilValue);
}

/* ----------------- Pointwise multiplication of sparse matrices */
SEXP mul_sparse(SEXP x, SEXP y){
  if(length(GET_SLOT(x, install("x"))) > length(GET_SLOT(y, install("x"))))
    return mul_sparse(y,x);
  CHM_SP A=AS_CHM_SP(x);
  CHM_SP B=AS_CHM_SP(y);
  int Am=A->nrow, Bm=B->nrow, An=A->ncol, Bn=B->ncol, Anzmax=A->nzmax, Bnzmax=B->nzmax;
  CHM_SP C=M_cholmod_allocate_sparse(Am, An, Anzmax,TRUE/*sorted*/,TRUE/*packed*/,
				     0 /*stype*/,CHOLMOD_REAL /*xtype*/,&c);
  int n=An,i,p;
  int *Ap=A->p,*Bp=B->p,*Cp=C->p,*Ai=A->i,*Bi=B->i,*Ci=C->i;
  double *Ax=A->x,*Bx=B->x,*Cx=C->x;
  double *w=Alloca(Bm,double);
  for(i=0;i<Bm;i++)w[i]=0.0;
  for(i=0;i<n;i++){
    for(p=Bp[i];p<Bp[i+1];p++)w[Bi[p]]=Bx[p];
    Cp[i]=Ap[i];
    for(p=Ap[i];p<Ap[i+1];p++){
      Cx[p]=Ax[p]*w[Ai[p]];
      Ci[p]=Ai[p];
    }
    for(p=Bp[i];p<Bp[i+1];p++)w[Bi[p]]=0.0;
  }
  Cp[An]=Anzmax; /* Finalize p */
  return M_chm_sparse_to_SEXP(C, 1, 0, 0, "", R_NilValue);
}


